<?php  
/* 
Plugin Name: OS OpenSpace Maps
Plugin URI: http://www.jonlynch.co.uk/wordpress-plugins/os-openspace-maps/ 
Description: Plugin for displaying OS OpenSpace Maps with elevation profiles 
Author: Jon Lynch 
Version: 1.2.2
Author URI: http://www.jonlynch.co.uk 
*/

// set up all our hooks
define("OSMAPS_VERSION", "1.2"); // set the version    
register_activation_hook( __FILE__, 'osmap_activate' ); // if the plugin has just been activated check for upgrade
add_filter('upload_mimes', 'my_myme_types', 1, 1);
add_action('admin_menu', 'osmap_admin_actions');  
add_action ('init', 'osmap_register_scripts');
add_action('wp_enqueue_scripts', 'add_my_stylesheet');
add_action('wp_footer', 'print_osmaps_scripts');
add_shortcode( 'osmap', 'osmap_generator');
add_action ('admin_head-post.php', 'osmap_add_help_tabs_to_post_page'); // add the help tabs to the post editing page
add_action ('admin_head-post-new.php', 'osmap_add_help_tabs_to_post_page');

// create the entry on the sidebar in the dashboard 
function osmap_admin_actions() {
    $osmaps_admin_page = add_options_page("OpenSpace Maps", "OpenSpace Maps", "manage_options", "OpenSpace_Maps", "osmap_admin"); 
    add_action('load-'.$osmaps_admin_page, 'osmaps_admin_add_help_tab');    
}

function osmap_add_help_tabs_to_post_page() {
	$screen = get_current_screen();
  $type = $screen->post_type;
	if ( ( $type !='post' )  && ( $type != 'page' ) ) return;
  if ( ! method_exists( $screen, 'add_help_tab' ) ) return; //for versions of WP before 3.3 TODO add contextual help
  $screen->add_help_tab( array(
    'id'      => 'osmap-help',
    'title'   => 'OS Maps',
    'content' => '<p>To add an OS OpenSpace map to your post or page use the shortcode <code>[osmap]</code>. The height and width can be set using <code>[osmap height="400" width ="500"]</code> where the dimensions are in pixels.</p><dl><dt>Markers</dt><dd>To add markers use the <em>markers</em> attribute. Add the marker location either as a 6, 8 or 10 figure grid reference or as a decimal lat and long. Each marker can be given a label by following the location with a semicolon and then adding the text. Separate each marker with a |. For example: <code>[osmap markers="NY21600720;Scafell Pike|NY18600880;Wasdale Head"]</code> will show two markers with the labels Wasdale and St Bees Head.</dd><dt>Tracks</dt><dd>To add tracks use the file uploader to upload kml or gpx files, then copy the address of the file. Then use the attribute <code>gpx="http://www.example.com/myfile.gpx"</code> or <code>kml="http://www.example.com/myfile.kml"</code> as appropriate. The tracks can be coloured using <code>colour="red"</code> or <code>colour="#ff0000"</code>.</dd></dl>',
  ) );  
  do_action( 'osmap_post_page_help');
};


function osmaps_admin_add_help_tab() {
  $screen = get_current_screen();
  if ( ! method_exists( $screen, 'add_help_tab' ) ) return; //for versions of WP before 3.3 TODO add contextual help
  $screen->add_help_tab( array(
    'id'      => 'overall-help',
    'title'   => 'Overview',
    'content' => '<p>This is where you set the options for using the OS Openspace Maps plugin.</p>',
  ) );  
  $screen->add_help_tab( array(
  'id'      => 'api-key',
  'title'   => 'API key',
  'content' => '<p>In order to use this plugin you need to register with the Ordnance Survey for an API key. Your key will be linked to your domain name and there will be a limit to the number of map tiles served to your API key each day. If you wish to use OpenSpace on multiple domains you can add them to your key.</p><p>Further information can he found on the <a href="http://openspace.ordnancesurvey.co.uk/openspace/">OS OpenSpace website</a>.</p>',
  ) );  
  $screen->add_help_tab( array(
  'id'      => 'map-defaults',
  'title'   => 'Map Defaults',
  'content' => '<dl><dt>Default Zoom Level</dt><dd>Set the default zoom level to be used when showing a single marker. Enter a number: 1 for the top resolution ticked below, 2 for the second etc. When a KML or GPX file is being used or multiple markers shown the map will zoom to fit.</dd><dt>Default Height</dt><dd>Set the default height of the map in pixels.</dd><dt>Default Width</dt><dd>Set the default width of the map in pixels.</dd><dt>Default Line Colour</dt><dd>Set the colour either using names eg <em>red</em> or using hex RGB codes eg <em>#ff0000</em></dd></dl>',
  ) );  
  $screen->add_help_tab( array(
    'id'      => 'resolutions',
    'title'   => 'Resolutions',
    'content' => '<p>Select all the resolutions you would like from the list, <strong>this applies to all maps</strong>.</p>',
    ) );    //TODO add more help
  do_action( 'osmap_settings_help');
};

// function to include our admin code from separate file
function osmap_admin() {  
  include('osmap_admin_options.php');  
}

function osmap_activate () {
  $key = osmap_api_key_check( get_option( 'osmap_options' )->apikey );
  if ( 'good' == $key ) {
    add_action ( 'admin_notices', 'osmap_good_api_key_notice');
  } elseif ( 'none' == $key ) {
    add_action ( 'admin_notices', 'osmap_no_api_key_notice');
  } else {
    add_action ( 'admin_notices', 'osmap_bad_api_key_notice');
  }
  //notices seem not to work when the plugin activated message is displayed
}

function osmap_good_api_key_notice () {?>
  <div class="updated"><p><strong><?php _e('OS Openspace Maps activated, API key working fine!' ); ?></strong></p></div>
  <?php
}

function osmap_bad_api_key_notice () {?>
  <div class="error"><p><strong>There appears to be an error with your API key.</strong> OS OpenSpace Maps will not be shown until it is corrected. To get an API key click <a href="https://openspace.ordnancesurvey.co.uk/osmapapi/register.do" title="API key registration">here</a>.</p><p>Further troubleshooting help is available <a href="http://osopenspacewiki.ordnancesurvey.co.uk/wiki/index.php?title=Troubleshooting">here</a>.</p></div>
 <?php
} 

function osmap_no_api_key_notice () {?>
    <div class="error"><p><strong>You have not entered an API key.</strong> OS OpenSpace Maps will not be shown until one is entered. To get an API key click <a href="https://openspace.ordnancesurvey.co.uk/osmapapi/register.do" title="API key registration">here</a>.</p><p>If you are developing on <code>localhost</code> you can enter anything.</p></div>
    <?php
}

// function that is run when the shortcode is found
function osmap_generator( $atts ) {
  // do nothing if we are on a feed
  if ( is_feed() ) return;
  /* get the options set on admin page */
  $options=get_option('osmap_options');
  // return if the api key is not valid
  if ( ! ( 'good' == $options->apikey_state ) ) return '<div id="os-map-error">OpenSpace Maps Error: Invalid API Key</div>';
  global $add_osmaps_scripts;
  $mapid = count( $add_osmaps_scripts );
  $add_osmaps_scripts[] = $mapid;
  $apikey = $options->apikey;  
  $default_zoom = $options->default_zoom;
  $default_height = $options->default_height;
  $default_width = $options->default_width;
  $default_color = $options->default_color;
  $res_2500 = $options->res_2500;
  $res_1000 = $options->res_1000;
  $res_500 = $options->res_500;
  $res_200 = $options->res_200;
  $res_100 = $options->res_100;
  $res_50 = $options->res_50;
  $res_25 = $options->res_25;
  $res_10 = $options->res_10;
  $res_5 = $options->res_5;
  $res_4 = $options->res_4;
  $res_3 = $options->res_3;
  $res_2 = $options->res_2;
  $res_1 = $options->res_1;
  /* get the shortcode info, set defaults */
  extract (shortcode_atts( array(
    'grid'   => '',
    'zoom'   => $default_zoom,
    'label'  => '',
    'height' => $default_height,
    'width'  => $default_width,
    'color'  => $default_color,
    'colour' => '',
    'll'     => '',
    'gpx'    => '',
    'kml'    => '',
    'markers'=> '',
  ), $atts ) );
  // use a proxy if the file is not on this host.
  $gpx = osmap_use_proxy( $gpx );
  $kml = osmap_use_proxy( $kml );
  // use colour in preference to color
  if ( $colour ) $color = $colour;
  // explode the markers into an array
  $markers = explode ( '|', $markers );
  //for backward compatibility add in grid marker.
  if ( $grid != '' ) {
    //TODO add a | if $markers has entries
    $markers[0] = $grid . ';' . $label;
  }  
if (''!=$width) 
  $width_css=' width: ' . $width . 'px;';
else 
  $width_css='';
$javascript = '
<script type="text/javascript">
  // <![CDATA[
  function initmap' . $mapid . '() {
  //initiate the map
  var options = {resolutions: [ ';
  if ($res_2500) {$javascript .='2500'; $comma=true;}
  if ($res_1000) {if ($comma) {$javascript .=', 1000';} else {$javascript .='1000';} $comma=true;}
  if ($res_500) {if ($comma) {$javascript .=', 500';} else {$javascript .='500';} $comma=true;}
  if ($res_200) {if ($comma) {$javascript .=', 200';} else {$javascript .='200';} $comma=true;}
  if ($res_100) {if ($comma) {$javascript .=', 100';} else {$javascript .='100';} $comma=true;}
  if ($res_50) {if ($comma) {$javascript .=', 50';} else {$javascript .='50';} $comma=true;}
  if ($res_25) {if ($comma) {$javascript .=', 25';} else {$javascript .='25';} $comma=true;}
  if ($res_10) {if ($comma) {$javascript .=', 10';} else {$javascript .='10';} $comma=true;}
  if ($res_5) {if ($comma) {$javascript .=', 5';} else {$javascript .='5';} $comma=true;}
  if ($res_4) {if ($comma) {$javascript .=', 4';} else {$javascript .='4';} $comma=true;}
  if ($res_3) {if ($comma) {$javascript .=', 2.5';} else {$javascript .='2.5';} $comma=true;}
  if ($res_2) {if ($comma) {$javascript .=', 2';} else {$javascript .='2';} $comma=true;}
  if ($res_1) {if ($comma) {$javascript .=', 1';} else {$javascript .='1';} ;}
  $javascript .= ']};
  osMap' . $mapid . ' = new OpenSpace.Map(\'singlemap-' . $mapid . '\', options);';
  if (''!=$gpx) { /* gpx file is specified */ $javascript .= '
    var gpxLoaded = function(event) {
          // \'this\' is layer
          this.map.zoomToExtent(this.getDataExtent());
      }
    osMap' . $mapid .'.addLayer(new OpenLayers.Layer.Vector("GPX Vectors", {
      style: {
        strokeColor: "' . $color . '",
        strokeWidth: 3,
        strokeOpacity: 0.8,
        pointRadius: 4,
        fillColor: "' . $color . '"
      },
      eventListeners: {
        "featuresadded": gpxLoaded
      },
      strategies: [new OpenLayers.Strategy.Fixed()],
      protocol: new OpenLayers.Protocol.HTTP({
        url: "' . $gpx . '",
        format: new OpenLayers.Format.GPX({
          internalProjection: new OpenLayers.Projection("EPSG:27700"),
          externalProjection: new OpenLayers.Projection("EPSG:4326")
        })
      })
    }));';
  } elseif (''!=$kml) { /*kml file is specified*/ $javascript .= '
    var gpxLoaded = function(event) {
          // \'this\' is layer
          this.map.zoomToExtent(this.getDataExtent());
      }
    osMap' . $mapid .'.addLayer(new OpenLayers.Layer.Vector("KML Vectors", {
      style: {
        strokeColor: "' . $color . '",
        strokeWidth: 3,
        strokeOpacity: 0.6,
        pointRadius: 4,
        fillColor: "' . $color . '"
      },
      eventListeners: {
        "featuresadded": gpxLoaded
      },
      strategies: [new OpenLayers.Strategy.Fixed()],
      protocol: new OpenLayers.Protocol.HTTP({
        url: "' . $kml . '",
        format: new OpenLayers.Format.KML({
          internalProjection: new OpenLayers.Projection("EPSG:27700"),
          externalProjection: new OpenLayers.Projection("EPSG:4326")
        })
      })
    }));';
  } else if ( $ll != '' ) {
    $javascript .= '
    var gridProjection = new OpenSpace.GridProjection();
    var lonlat = new OpenLayers.LonLat(' . $ll .');
    var mapPoint = gridProjection.getMapPointFromLonLat(lonlat);
    osMap' . $mapid . '.setCenter( mapPoint, ' . $zoom . ');
    var markers = new OpenLayers.Layer.Markers("Markers");
    osMap' . $mapid . '.addLayer(markers);
    //add a single marker containing information on the current post
    var content = "' . $label . '";
    var marker = osMap' . $mapid . '.createMarker(mapPoint, null, content);
    markers.addMarker(marker); ';
  } else {
    if ( $markers[0] != '' ) {
      // centre on first marker
      $centre = explode( ';', $markers[0]);
      $centre = $centre[0];
      $javascript .= '
        osMap' . $mapid . '.setCenter( NGR2NE ("' . $centre . '"), ' . $zoom . ');
        var markers = new OpenLayers.Layer.Markers("Markers");
        osMap' . $mapid . '.addLayer(markers);
        ';
     
      foreach ( $markers as $marker ) {
        $bits = explode( ';', $marker );
        $javascript .= '
        var pos = CONVERT ("' . $bits[0] . '"); 
        var content = "' . $bits[1] . '";
        var marker = osMap' . $mapid . '.createMarker(pos, null, content);
        markers.addMarker(marker); 
        ';
      }
    }
  }
  // zoom to fit markers if more than 1
  if ( count( $markers ) > 1 ) {
   $javascript .='
        osMap' . $mapid . '.zoomToExtent(osMap' . $mapid . '.getMarkerLayer().getDataExtent());';
  }    
  // disable keyboard scrolling when not over the map
  $javascript .= '
    for( var c in osMap' . $mapid . '.controls )
      if( osMap' . $mapid . '.controls[c].CLASS_NAME == "OpenLayers.Control.KeyboardDefaults" )
      {
        keys = osMap' . $mapid . '.controls[c];
        osMap' . $mapid . '.events.register( \'mouseover\', this, function(){ keys.activate(); } );
        osMap' . $mapid . '.events.register( \'mouseout\', this, function(){ keys.deactivate(); } );
        keys.deactivate();
      }
  }
  // ]]> 
</script>';


  $javascript .= '<div class="osmap" id="singlemap-' . $mapid . '" style="display:none; height:' . $height . 'px; max-width:100%;' . $width_css . '"></div><div id="noscriptmap-' . $mapid . '"><h5>To see this map cookies and javascript must be enabled. If you are still having trouble after having checked both of these please contact us using the link at the top of the page</h5></div><script type="text/javascript">
    document.getElementById("singlemap-' . $mapid . '").style.display = "block";
    document.getElementById("noscriptmap-' . $mapid . '").style.display = "none"' . '</script><div id="spacer"></div>';
  $javascript = apply_filters ('osmap_output', $javascript, $atts, $mapid  );
  return $javascript; 
}

//register scripts
function osmap_register_scripts() {
$plugin_dir_path = plugins_url() . '/os-openspace-maps';  
  wp_register_script ("openspace",
    'http://openspace.ordnancesurvey.co.uk/osmapapi/openspace.js?key='.get_option('osmap_options')->apikey,
    "",
  "1.0",
    false); 
  wp_register_script ('osmapsjs', 
    $plugin_dir_path . '/osmapsjs.js',
    '',
    '1.0',
    true);
}

function print_osmaps_scripts() {
  //checks if they are needed
  global $add_osmaps_scripts;
  if ( ! $add_osmaps_scripts )
    return; 
  wp_print_scripts('openspace');
  wp_print_scripts('osmapsjs');?>
  <script type="text/javascript">
  <?php
    foreach ( $add_osmaps_scripts as $mapno ) {
    
        echo "if(typeof initmap". $mapno . " == 'function') initmap". $mapno . "();";
    }
    ?>
  </script>
  <?php
}

// Enqueue style-file, if it exists.
function add_my_stylesheet() {
    $myStyleUrl = WP_PLUGIN_URL . '/os-openspace-maps/style.css';
    $myStyleFile = WP_PLUGIN_DIR . '/os-openspace-maps/style.css';
    if ( file_exists($myStyleFile) ) {
        wp_register_style('my_maps_stylesheet', $myStyleUrl);
        wp_enqueue_style( 'my_maps_stylesheet');
    }
}
  
function my_myme_types($mime_types){
  $mime_types['gpx'] = 'text/xml'; //Adding gpx extension
  $mime_types['kml'] = 'text/xml'; //Adding kml extension
  return $mime_types;
}

// check for a valid API key, returns none, good or bad
function osmap_api_key_check( $key ) {
  // if there is no API key
  if ( ! $key ) {
    return 'none';
  }
  // there is a key we now need to check it
  $url = 'http://openspace.ordnancesurvey.co.uk/osmapapi/openspace.js?key=';
  $args = array();
  $args[timeout]=15;
  $result = wp_remote_get( $url . $key, $args );
  if( is_wp_error( $result ) )
    return 'error';
  if ( 200 == $result['response']['code'] ) {
    return 'good';
  } else {
    return 'bad';
  }  
}

function osmap_use_proxy ( $file ) {
  if ( ( ! strripos ( $file , $_SERVER['SERVER_NAME'] ) ) && $file )
    $file = plugin_dir_url( __file__ ) . 'proxy.php?url=' . $file;
  return $file;
}

?>